/*
 * Copyright 2002-2007 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.json.util;

import net.sf.json.JSONObject;

/**
 * Base class for creating Bean instances.<br>
 * <ul>
 * <li>DEFAULT - calls Class.newInstance().</li>
 * </ul>
 *
 * @author Andres Almiray <aalmiray@users.sourceforge.net>
 */
public abstract class NewBeanInstanceStrategy {
   /** Calls Class.newInstance() */
   public static final NewBeanInstanceStrategy DEFAULT = new DefaultNewBeanInstanceStrategy();

   /**
    * Creates a new instance.
    *
    * @param target the source class
    * @param target additional properties that may be needed to create the
    *        instance
    */
   public abstract Object newInstance( Class target, JSONObject source )
         throws InstantiationException, IllegalAccessException;

   private static final class DefaultNewBeanInstanceStrategy extends NewBeanInstanceStrategy {
      public Object newInstance( Class target, JSONObject source ) throws InstantiationException,
            IllegalAccessException {
         if( target != null ){
            return target.newInstance();
         }
         return null;
      }
   }
}